/*==========================================================================
 =                                                                         =
 =                        Project CTI-Print                                =
 =                                                                         =
 = Author:                                                                 =
 =   Panos Dimakopoulos, Systems Programmer,                               =
 =   Computer Technology Institute,                                        =
 =   Division of Computing Facilities,                                     =
 =   P.O. Box 1122,                                                        =
 =   261 10  Patras,                                                       =
 =   Greece                                                                =
 =   (e-mail: dimakop@cti.gr)                                              =
 =   Tel: +30 61 992061                                                    =
 =   Fax: +30 61 993973                                                    =
 =                                                                         =
 = Created by Patrick Powell  <papowell@sdsu.edu>                          =
 =   for LPRng software Sat Aug 26 06:54:25 PDT 1995                       =
 ==========================================================================*/


/*==========================================================================
  Version CTI-LPRng-1.0
  =========================================================================*/


/*
 * $Id: args.c,v 1.10 1996/07/19 23:40:06 papowell Exp $
 */

#include "portable.h"
#include "hp4.h"
#include "common.h"

static char *tflag;

struct parm Parmlist[] = {
{'C', &class, STRV },
{'F', &format, STRV },
{'J', &job, STRV },
{'L', &username, STRV },
{'N', &filename, STRV },
{'P', &printer, STRV },
{'R', &accntname, STRV },
{'S', &pr_commnt, STRV },
{'T', &tflag, STRV },
{'X', &seq_number, STRV },
{'Z', &zopts, STRV },
{'c', (char **)&literal, FLGV },
{'h', &host, STRV },
{'i', (char **)&indent, INTV },
{'k', &controlfile, STRV },
{'l', (char **)&length, INTV },
{'n', &login, STRV },
{'s', &statusfile, STRV },
{'v', (char **)&version, FLGV },
{'w', (char **)&width, INTV },
{'x', (char **)&xwidth, INTV },
{'y', (char **)&ylength, INTV } };

int Parmlen = sizeof(Parmlist)/sizeof(Parmlist[0]);

struct parm *findvar( int flag, struct parm *parmlist, int parmlen );
void setvar(  struct parm *parm, char *arg );

static struct value Valuelist[] = {
	{ "accounting", &Accounting_script, STRV },
	{ "autodetect", (char **)&autodetect, FLGV },
	{ "banner", (char **)&bnr, FLGV },
	{ "cartridge", (char **)&cartridge, FLGV },
	{ "debug", (char **)&debug, INTV },
	{ "dev", &device, STRV },
	{ "logall", &logall, FLGV },
	{ "model", &model_name, STRV },
	{ "pagecount", (char **)&pagecount, FLGV },
	{ "plp", (char **)&plp_compat, FLGV },
	{ "resourcesave", (char **)&resourcesave, FLGV },
	{ "retries", (char **)&retries, INTV },
	{ "sleep", (char **)&wait_time, INTV },
	{ "status", (char **)&get_status, FLGV },
	{ "stty", &stty_args, STRV },
	{ "summary", &summaryfile, STRV },
	{ "title", &bantitle, STRV },
	{ "trace", (char **)&trace, FLGV },
	{ "wrap", (char **)&wrap, FLGV },
};
int Valuelen = sizeof(Valuelist)/sizeof(Valuelist[0]);

extern int getopt(), optind;
extern char *optarg;

void getargs(argc, argv)
	int argc;
	char **argv;
{
	int i;		/* argument index */
	int flag;	/* flag */
	char *arg;	/* argument */
	struct parm *parm;

	name = argv[0];

	for( i = 1; i < argc; ++i ){
		arg = argv[i];
		if( *arg++ != '-' ) break;
		flag = *arg++;
		parm = findvar( flag, Parmlist, Parmlen );
		/* not found, assume -X argument format */
		if( parm == 0 || parm->kind != FLGV ){
			if( flag == 0 || *arg == 0 ){
				/* skip to next option */
				arg = argv[++i];
			}
		}
		if( parm == 0 ) continue;
		if( flag != 'T' ){
			setvar( parm, arg );
		} else {
		/*
		 * -T[dev=device][,model=(IV|II|III|IIIsi)][,CART,STTY=str]
		 */
			char *end, *value;
			for( ; arg && *arg; (arg = end) ){
				if( (end = strchr( arg, ',' )) ){
					*end++ = 0;
				}
				while( isspace(*arg) ) ++arg;
				if( (value = strchr( arg, '=' )) ){
					*value++ = 0;
				}
				setvalue( arg, value, Valuelist, Valuelen );
			}
		}
	}
	if( i < argc ) accntfile = argv[i];
	newstatus();
	log(5,"literal %d", literal );
}

/*
 * setvar(  struct parm *parm, char *arg )
 * 1. if STRV, then set 
 * 2. if INTV, then convert and set
 * 3. if FLGV, then set to 1
 */

void setvar( struct parm *parm, char *arg )
{

	switch( parm->kind ){
	case STRV: *parm->var = arg; break;
	case INTV: *(int *)parm->var = atoi(arg); break;
	case FLGV: *(int *)parm->var = 
		(arg == 0 || *arg == 0 || !strcasecmp( arg, "yes" )
			|| !strcasecmp( arg, "on" ));
		break;
	}
}

struct parm *findvar( int flag, struct parm *parmlist, int parmlen  )
{
	int u, l, i, c;	/* upper, lower, i */

	l = 0; u = parmlen;
	while( l <= u ){
		i = (u+l)/2;
		c = flag - parmlist[i].flag;
		if( 0 == c ){
			log(4, "found parm %c, %d", flag, i );
			return( &parmlist[i] );
		} else if( c < 0 ){
			 log(6, "down parm %c, %d", flag, i );
			u = i - 1 ;
		} else {
			 log(6, "up parm %c, %d", flag, i );
			l = i + 1 ;
		}
	}
	logerr(5,"Invalid option -%c", flag );
	return(0);
}

void setvalue( flag, arg, valuelist, valuelen )
	char *flag;
	char *arg;
	struct value *valuelist;
	int valuelen;
{
	int u, l, i, c;	/* upper, lower, i */

	l = 0; u = valuelen-1;
	while( l <= u ){
		i = (u+l)/2;
		c = strcasecmp( flag,valuelist[i].flag);
		if( 0 == c ){
			log(4, "found option %s, [%d], arg '%s'", flag, i, arg );
			switch( valuelist[i].kind ){
			case STRV: *valuelist[i].var = arg; break;
			case INTV: *(int *)valuelist[i].var = atoi(arg); break;
			case FLGV: *(int *)valuelist[i].var =
				(arg == 0 || !strcasecmp( arg, "yes")
					|| !strcasecmp( arg, "on" ));
				break;
			}
			return;
		} else if( c < 0 ){
			log(6, "down parm %s, %d", flag, i );
			u = i - 1 ;
		} else {
			log(6, "up parm %s, %d", flag, i );
			l = i + 1 ;
		}
	}
	fprintf( stderr, "Invalid -T flag '%s', valid flags are:", flag );
	for( i = 0; i < valuelen; ++i ){
		char *s;
		if( i % 4 ){
			fprintf( stderr, ", " );
		} else {
			fprintf( stderr, "\n   " );
		}
		switch( valuelist[i].kind ){
			case STRV: s = "str"; break;
			case INTV: s = "num"; break;
			case FLGV: s = "[on|off]"; break;
			default: s = "???"; break;
		}
		fprintf( stderr, "%s=%s", valuelist[i].flag, s );
	}
	fprintf( stderr, "\n" );
	errorcode = JABORT;
	fatal("Invalid -T flag '%s'", flag );
}
